#include	"lpc210x.h"
#include	"glcd.h"
#include	"uart.h"

unsigned char linebuf[GRAPHIC_BYTESPERROW];
unsigned char outp[GRAPHIC_BYTESPERROW];


unsigned char LCD_Get(unsigned char index){
	int li;
//	LCDDATA_CLR;
	LCDIODATA_IN;
	LCDIOSET |= ((1<<WR)|(1<<RD)|(1<<CE));
	if(index=='s')LCDIOSET |= (1<<CD); else LCDIOCLR |= (1<<CD);
	LCDIOCLR |= (1<<RD)|(1<<CE);
	for(li=0;li<(2*LCDDELAY);li++)asm("nop");
	li |= LCDDATA_PIN;
	LCDIOSET |= ((1<<WR)|(1<<CE)|(1<<RD));
	return (unsigned char) li;
}

void LCD_Send(unsigned char index,unsigned char data){
	int li;
	while((LCD_Get('s')&3) != 3);
	LCDIODATA_OUT;
	LCDDATA_CLR;
	LCDIOSET |= ((1<<WR)|(1<<RD)|(1<<CE));
	if(index=='c') LCDIOSET |= (1<<CD); else LCDIOCLR |= (1<<CD);
	LCDDATA(data);
	LCDIOCLR |= ((1<<WR)|(1<<CE));
	for(li=0;li<LCDDELAY;li++)asm("nop");
	LCDIOSET |= ((1<<WR)|(1<<CE)|(1<<RD));
	for(li=0;li<LCDDELAY;li++)asm("nop");
	LCDDATA_CLR;
}

void LCD_Send2(unsigned char index,unsigned char data, unsigned char checkval){
	int li;
	while((LCD_Get('s')&checkval) != checkval);
	LCDIODATA_OUT;
	LCDDATA_CLR;
	LCDIOSET |= ((1<<WR)|(1<<RD)|(1<<CE));
	if(index=='c') LCDIOSET |= (1<<CD); else LCDIOCLR |= (1<<CD);
	LCDDATA(data);
	LCDIOCLR |= ((1<<WR)|(1<<CE));
	for(li=0;li<LCDDELAY;li++)asm("nop");
	LCDIOSET |= ((1<<WR)|(1<<CE)|(1<<RD));
	for(li=0;li<LCDDELAY;li++)asm("nop");
	LCDDATA_CLR;
}

void LCD_SendPacket(unsigned char cmd, unsigned char d1, unsigned char d2){
	LCD_Send('d',d1);
	LCD_Send('d',d2);
	LCD_Send('c',cmd);	
}

void LCD_SendData(unsigned char data){
	LCD_Send('d',data);
	LCD_Send('c',192);
}

unsigned char LCD_ReadData(void){
	LCD_Send('c',197);
	return LCD_Get('d');
}

void LCD_ClearRam(unsigned int ptr, unsigned int size){
	LCD_SetAddrPtr(ptr);
	while(size--)LCD_SendData(0);
}

void LCD_SetHomePtr(unsigned char gt, unsigned int ptr)
{
	LCD_SendPacket(SETCONTWORD|((gt=='g')?2:0),(unsigned char)(ptr&0xFF),(unsigned char)(ptr>>8));
}

void LCD_SetAddrPtr(unsigned int ptr)
{
	LCD_SendPacket(REGISTERSET|SETADDRPTR,(unsigned char)(ptr&0xFF),(unsigned char)(ptr>>8));
}

void LCD_SetCursorPtr(unsigned char column, unsigned char row){
	LCD_Send('d',column);
	LCD_Send('d',row);
	LCD_Send('c',REGISTERSET|SETCURSORPTR);
}

void LCD_SetCustomCharacters(void){
        LCD_Send('d',0x03);
        LCD_Send('d',0x00);
        LCD_Send('c',REGISTERSET|SETOFFSETREG);

        LCD_SetAddrPtr(0x1C00);
        LCD_SendData(0x0E);		//ß
        LCD_SendData(0x11);
        LCD_SendData(0x11);
        LCD_SendData(0x12);
        LCD_SendData(0x11);
        LCD_SendData(0x11);
        LCD_SendData(0x16);
        LCD_SendData(0x10);

        LCD_SendData(0x0A);		//Ü
        LCD_SendData(0x00);
        LCD_SendData(0x11);
        LCD_SendData(0x11);
        LCD_SendData(0x11);
        LCD_SendData(0x11);
        LCD_SendData(0x0E);
        LCD_SendData(0x00);

        LCD_SendData(0x18);		//°
        LCD_SendData(0x18);
        LCD_SendData(0x07);
        LCD_SendData(0x08);
        LCD_SendData(0x08);
        LCD_SendData(0x08);
        LCD_SendData(0x07);
        LCD_SendData(0x00);


}

void LCD_Init(void){
	int i;
	
	LCDDATA_PINSEL0 &= ~(LCDDATA_PSMASK0);
	LCDDATA_PINSEL1 &= ~(LCDDATA_PSMASK1);
	LCDCFG_PINSEL   &= ~(LCDCFG_PSMASK);

	LCDIODATA_OUT;
	LCDIODIR |= ((1<<WR)|(1<<RD)|(1<<CE)|(1<<CD)|(1<<RES));
	LCDIOSET |= ((1<<WR)|(1<<RD)|(1<<CE)|(1<<CD)|(1<<RES));

	LCDIOCLR |= (1<<RES);							// Reset LCD
	for(i=0;i<64000;i++) { asm("nop");asm("nop");asm("nop"); }		// ...
	LCDIOSET |= (1<<RES);							// ...
	for(i=0;i<64000;i++) asm("nop");					// ...
	while((LCD_Get('s')&3) != 3);						// ...
	for(i=0;i<64000;i++) asm("nop");					// Reset LCD


	LCD_Send('c',MODESET|EXORMODE|CGROMMODE);				// Set Textmode 'OR' and CGROM-Mode
	LCD_SendPacket(SETCONTWORD|GRAPHICAREA,GRAPHIC_BYTESPERROW,0);		// Set amount of bytes per graphic row
	LCD_SendPacket(SETCONTWORD|TEXTAREA,TEXT_BYTESPERROW,0);		// Set amount of bytes per text row
	LCD_ClearRam(0x0000,0x18C0);						// clear the first 7kb
	LCD_Send('c',DISPLAYMODE|TEXTON|GRAPHICON);				// Switch Graphic and Textdisplay on
	LCD_Send('c',CURSORPATTERN|CURSOR1LINE);				// Set Cursor to 1 Line height

	LCD_SetCustomCharacters();
}


void LCD_SendLine(unsigned char count){
	unsigned char i;
	LCD_Send('c',DATAAUTORDWR|SETAUTOWRITE);
	for(i=0;i<count;i++)
		LCD_Send2('d',outp[i],8); 
	while((LCD_Get('s')&8)!=8);
	LCD_Send('c',DATAAUTORDWR|AUTORESET);
}

void LCD_GetLine(unsigned char count){
	unsigned char i;
	LCD_Send('c',DATAAUTORDWR|SETAUTOREAD);
	for(i=0;i<count;i++) 
	{ 
		while((LCD_Get('s')&4)!=4);
		outp[i]=LCD_Get('d');
	}
	while((LCD_Get('s')&4)!=4);
	LCD_Send('c',DATAAUTORDWR|AUTORESET);
}

void LCD_ModLine(unsigned int ptr, unsigned char count, unsigned char mod)
{
	unsigned char t;
	LCD_SetAddrPtr(ptr);
	LCD_GetLine(count);
	for (t=0; t<GRAPHIC_BYTESPERROW; t++)
	{
		if (mod=='a') outp[t]&=linebuf[t];	
		else if (mod=='o') outp[t]|=linebuf[t];	
		else if (mod=='x') outp[t]^=linebuf[t];	
	}
	LCD_SetAddrPtr(ptr);
	LCD_SendLine(count);
}
