/*
Sample for RAM mapping:
0x0000			<- GHP
...	Graphic Page 0
0x0AFF
0x0B00			<-GHP
...	Graphic Page 1
0x15FF
0x1600			<-THP
...	Text Page 0
0x175F			
0x1760			<-THP
...	Text Page 1
0x18C0

Only works in CG ROM Mode. CG RAM Mode needs the last 2kb of controller ram.
*/

#ifndef GLCD_H
#define GLCD_H

#include	"lpc210x.h"

#define LCDDELAY	1

#define LCDDATA_PINSEL0	PINSEL0
#define LCDDATA_PSMASK0	0xFFC00000
#define LCDDATA_PINSEL1	PINSEL1
#define LCDDATA_PSMASK1	0x0000003F
#define	LCDCFG_PINSEL	PINSEL1
#define LCDCFG_PSMASK	((1<<RES)|(1<<CD)|(1<<CE)|(1<<RD)|(1<<WR))
#define LCDIODIR	IODIR
#define LCDIOCLR	IOCLR
#define LCDIOSET	IOSET
#define LCDIOPIN	IOPIN
#define LCDDATA_OFFSET 	11	
#define	LCDDATA_MASK	0x0007F800

#define	RES		27	//Reset
#define CD		26	//C/!D
#define CE		25	//!CE
#define RD		29	//!RD
#define WR		28	//!WR

#define	LCDIODATA_OUT	LCDIODIR |= LCDDATA_MASK
#define LCDIODATA_IN	LCDIODIR &= ~(LCDDATA_MASK)
#define LCDDATA_CLR	LCDIOCLR |= (0xFF<<LCDDATA_OFFSET)
#define LCDDATA(data)	LCDIOSET |= (((unsigned char)data)<<LCDDATA_OFFSET) 
#define LCDDATA_PIN	(unsigned char) ((LCDIOPIN & LCDDATA_MASK)>>LCDDATA_OFFSET)

#define GP0				0x0000
#define GP1				0x0A00
#define TP0				0x1400
#define TP1				0x1540
#define GRAPHIC_BYTESPERROW		22		//Graphic Area
#define TEXT_BYTESPERROW		22		//Text Area
#define GRAPHIC_HEIGHT			64		//Graphic Height
#define	TEXT_HEIGHT			8		//Text Height
#define BITPERBYTE			6		//Used Bits per Byte 


#define REGISTERSET	0x20
#define		SETCURSORPTR	0x01
#define		SETOFFSETREG	0x02
#define		SETADDRPTR	0x04
#define	SETCONTWORD	0x40
#define 	TEXTHOMEADDR	0x00
#define 	TEXTAREA	0x01
#define		GRAPHICHOMEADDR	0x02
#define		GRAPHICAREA	0x03
#define	MODESET		0x80
#define		ORMODE		0x00
#define		EXORMODE	0x01
#define		ANDMODE		0x02
#define		TEXTATTRMODE	0x03
#define		CGROMMODE	0x00
#define		CGRAMMODE	0x08
#define	DISPLAYMODE	0x90
#define		DISPLAYOFF	0x00
#define		BLINKON		0x01
#define		CURSORON	0x02
#define		TEXTON		0x04
#define		GRAPHICON	0x08
#define	CURSORPATTERN	0xA0
#define		CURSOR1LINE	0x00
#define		CURSOR2LINE	0x01
#define		CURSOR3LINE	0x02
#define		CURSOR4LINE	0x03
#define		CURSOR5LINE	0x04
#define		CURSOR6LINE	0x05
#define		CURSOR7LINE	0x06
#define		CURSOR8LINE	0x07
#define	DATAAUTORDWR	0xB0
#define		SETAUTOWRITE	0x00
#define		SETAUTOREAD	0x01
#define		AUTORESET	0x02
#define	DATARDWR	0xC0
#define		WRITEINCADP	0x00
#define		READINCADP	0x01
#define		WRITEDECADP	0x02
#define		READDECADP	0x03
#define		WRITECONSTADP	0x04
#define		READCONSTADP	0x05
#define	SCREENPEEK	0xE0
#define	SCREENCOPY	0xE8
#define	BITSETRESET	0xF0
#define		BITRESET	0x00
#define		BITSET		0x08
#define		BIT0		0x00
#define		BIT1		0x01
#define		BIT2		0x02
#define		BIT3		0x03
#define		BIT4		0x04
#define		BIT5		0x05
#define		BIT6		0x06
#define		BIT7		0x07


extern unsigned char linebuf[GRAPHIC_BYTESPERROW];


void LCD_Init(void);

/*
 *	Daten zum Display schreiben
 *	index:
 *	'c'	Command
 *	'd'	Data
 *	data:
 *	8-bit LCD-Data
 */
void LCD_Send(unsigned char index, unsigned char data);
void LCD_Send2(unsigned char index,unsigned char data, unsigned char checkval);

/*
 *	Daten vom Display lesen
 *	index:
 *	's'	Status
 *	'd'	Data
 *	return:
 *	data
 */
unsigned char LCD_Get(unsigned char index);

/*
 *	Home Address Pointer
 *	gt:
 *	't'	Text
 *	'g'	Graphic
 *	ptr:
 *	16-bit Pointer
 */
void LCD_SetHomePtr(unsigned char gt, unsigned int ptr);

/*
 *	Daten Pointer setzen
 *	ptr:
 *	16-bit Pointer
 */
void LCD_SetAddrPtr(unsigned int ptr);

void LCD_SetCursorPtr(unsigned char column, unsigned char row);

/*
 *	Daten ins RAM schreiben, mit anschlieender AddrPtr++
 *	data:
 *	8-bit Data
 */ 
void LCD_SendData(unsigned char data);

/*
 * 	Daten aus dem RAM lesen, ohne AddrPtr++
 * 	return:
 * 	8-bit Data
 */
unsigned char LCD_ReadData(void);

/*
 *	Ram aud 0x00 setzen
 *	ptr:
 *	16-bit Startwert
 *	size:
 *	16-bit Gre des Bereichs
 */
void LCD_ClearRam(unsigned int ptr, unsigned int size);

/*
 *	Eine Zeile auf das Display bertragen
 *	mode:
 *	'g'	Graphic Linie	(16-40byte Array)
 *	't'	Text Zeile	(21-40byte Array)
 *	*line:
 *	Array bestehend aus  unsigned chars
 */
void LCD_SendLine(unsigned char count);

/*
 *	Eine Zeile aus dem Dispaly auslesen
 *	mode:
 *	'g'	Graphic Linie	(16-40byte Array)
 *	't'	Text Zeile	(21-40byte Array)
 *	return:
 *	unsigned char Pointer to Array
 */
void LCD_GetLine(unsigned char count);

/*
 *	Eine Zeile modifizieren - Graphic only
 *	ptr:
 *	16-bit Startwert
 *	mode:
 *	'a'	AND Mode
 *	'o'	OR Mode
 *	'x'	XOR Mode
 *	line:
 *	unsigend char Pointer to Array
 */
void LCD_ModLine(unsigned int ptr, unsigned char count, unsigned char mod);

#endif
