// FR 6x8 FONT!!!!

#include "lpc210x.h"
#include <stdarg.h>
#include <ctype.h>
#include <string.h>
#include "glcd.h"


typedef unsigned char  BYTE;
typedef unsigned short WORD;


// Hier bilden wir eine Bitmaske zum Testen des UDRE-Bits im USR Register
//const BYTE mskUdrelcd = 1 << UDRE;
 

// Funktionsprototyp der lokalen Hilfsfunktion zur Ausgabe eines Zeichens
static void glcdwritebuf (BYTE *pBuf, int nLen);
static void glcdwrite (BYTE c);


// Hilfsvariable zum Zhlen der ausgegebenen Zeichen
int		nSent;
BYTE     	nFieldWidth;

static struct {
    BYTE     fLeftJust:1;   // Feldausrichtung links oder rechts
    BYTE     fNegative:1;   // Auszugebende Zahl ist negativ.
    BYTE     last:1;
} flags;


int glcdprintf (unsigned int ptrin, unsigned char row, unsigned char column, const unsigned char *format, ...)
/**********************************************************************
ptrin = RAM Ptr
row = Zeile
column = Spalte


    Einfache Routine, um formatierte Ausgaben auf den GLCD zu schicken.
    Der Vorteil dieser Routine gegenber einem sprintf ist dass hier
    kein kostbarer Speicherplatz im SRAM verschwendet wird sondern
    das Resultat wird direkt mit dem GLCD ausgegeben.
    Der GLCD muss zuvor korrekt initialisiert werden.

    in   : format    Formatstring, wie von printf her bekannt.
                     Es werden folgende Platzhalterzeichen und Flags
                     erkannt.
                     %c     Einzelnes Zeichen, erwartet char-Argument.
                     %s     Zeichenkette, erwartet char *-Argument
                     %d,%i  Ganzzahl mit Vorzeichen,
                            erwartet int-Argument.
                     %u     Ganzzahl ohne Vorzeichen,
                            erwartet int-Argument
                     %x,%X  Hexzahl (ohne fhrendes 0x),
                            erwartet int-Argument.
                     %o     Oktalzahl (ohne fhrendes 0),
                            erwartet int-Argument
                     Ein - nach dem %-Zeichen bedeutet, dass die Ausgabe
                     linksbbndig ausgerichtet ist.
                     Ein Zahlenwert nach dem % bzw. dem - gibt die
                     Breite des Ausgabefeldes an. Die Ausgabe erfolgt
                     falls notwendig auch mit mehr Zeichen.
*/
{
	flags.last=0;
	#define SCRATCH 16
    	BYTE     scratch[SCRATCH];
    	BYTE     format_flag;
    	WORD     u_val=0, base;
    	BYTE     *ptr;
    	va_list  ap;
    	BYTE     hexA = 'a';

    	nSent = column;

	LCD_SetAddrPtr(ptrin+(row*GRAPHIC_BYTESPERROW)+column);
    va_start (ap, format);
    for (;;) {
        /* In der folgenden Schleife werden alle Zeichen direkt
           weitergegeben, sofern es sich nicht um ein Platzhalterzeichen
           '%' handelt. */
        while ((format_flag = *format++) != '%') {
            if (!format_flag) {   // Ende des Formatstrings erreicht
                va_end (ap);
                return (nSent-column);
            }
            // Ende des Strings noch nicht erreicht
            // und kein Platzhalterzeichen, also ausgeben
            if(format_flag==223){							// wird durch ss ersetzt
            	glcdwrite ('s');
            	glcdwrite ('s');
            } else glcdwrite (format_flag);
        }


        // Mit %10s kann beispielsweise ein 4 Zeichen langer String in
        // einem 10 Zeichen langen Feld ausgegeben werden. Zustzlich
        // kann mit einem - Zeichen die Ausrichtung angegeben werden.
        // glcdprintf ("%10s", "Hallo")  ergibt "     Hallo"
        // glcdprintf ("%-10s", "Hallo") ergibt "Hallo     "
        // Desweiteren kann mit %05d angegeben werden, dass ein Zahlwenwert
        // in einem rechtsbndigen Feld der Lnge 5 ausgegeben wird wobei
        // links mit Nullen aufgefllt wird.
        format_flag = *format++;
        if (format_flag == '-') {
            flags.fLeftJust = 1;
            format_flag = *format++;
        }
        else flags.fLeftJust = 0;

        nFieldWidth = 0;
        while (isdigit (format_flag)) {
           nFieldWidth *= 10;
           nFieldWidth += format_flag - '0';
           format_flag = *format++;
        }

        switch (format_flag) {
            case 'c':
                format_flag = va_arg (ap, int);
            default:
                glcdwritebuf (&format_flag, 1);
                continue;
            case 's':
                ptr = va_arg (ap, char *);
                glcdwritebuf (ptr, -1);
                continue;
            case 'o':
                base = 8;
                goto CONVERSION_LOOP;
            case 'i':
            case 'd':
            case 'u':
                base = 10;
                goto CONVERSION_LOOP;
            case 'X':
                hexA = 'A';
                // Weiter wie 'x'
            case 'x':
                base = 16;
CONVERSION_LOOP:
                u_val = va_arg (ap, int);
                flags.fNegative = 0;
                if (format_flag == 'd' || format_flag == 'i') {
                    // Negative Werte auswerten
                    if (((int) u_val) < 0) {
                        flags.fNegative = 1;
                        u_val = -u_val;
                    }
                }

                // Der Scratchpuffer wird von rechts nach links aufgefllt
                // beginnend mit dem niederwertigsten Digit.
                ptr = scratch + SCRATCH;
                *--ptr = 0;     // Abschliessendes NULL-Byte eintragen
                do {
                    char ch = u_val % base + '0';
                    if (ch > '9') ch += hexA - '9' - 1;
                    *--ptr = ch;
                    u_val /= base;
                } while (u_val);

                if (flags.fNegative) *--ptr = '-';
                glcdwritebuf (ptr, -1);    // Scratchpuffer ausgeben
        }
     }

} /* end of glcdprintf */



static void glcdwritebuf (BYTE *pBuf, int nLen)
/**********************************************************************
    Schreibt einen Puffer auf den GLCD. Dabei werden die Feldbreite und
    -ausrichtung bercksichtigt.

   in   : pBuf   Adresse der auszugebenden Daten.
   in   : nLen   Anzahl auszugebende Zeichen. Wird hier -1 angegeben
                 so wird ausgegeben bis ein NULL-Zeichen gefunden wird.
*/
{
    BYTE *p;
    int   n;

    if (nLen == -1) {
       // Lnge bestimmen
       p = pBuf;
       nLen = 0;
       while (*p++) nLen++;
    }

    // Feld bei Bedarf links auffllen
    if (!flags.fLeftJust) {
        for (n=nLen; n < nFieldWidth; n++) glcdwrite (' ');
    }

    // Pufferinhalt schreiben
    for (n=0; n < nLen; n++) glcdwrite (*pBuf++);

    // Feld bei Bedarf rechts auffllen
    if (flags.fLeftJust) {
        for (n=nLen; n < nFieldWidth; n++) glcdwrite (' ');
    }

} /* end of Blank */



static void glcdwrite (BYTE c){
	int temp=nSent;
	while(temp>=TEXT_BYTESPERROW)temp-=TEXT_BYTESPERROW;
	if((temp==0)&&(nSent!=0)&&(flags.last==0)){
		//lcddata(0); 
	} else if(temp!=0)flags.last=0;
	switch(c){
		case '\n':
			temp=TEXT_BYTESPERROW-temp;
			while(temp){
				LCD_SendData(0);
				nSent++;
				temp--;
			}
			nSent--;
			break;
    		case ' ':
    			if((temp==0)&&(nSent!=0)&&(flags.last==0)){
    				nSent--; 
    				flags.last=1;
    			} else LCD_SendData(0);
    			break;
     		default:
    			LCD_SendData(c-0x20);
	}
    nSent++;

}
