#ifndef __SYSTEM_H
#define __SYSTEM_H

#ifndef BIT
#define BIT(n)              (1 << (n))
#endif

/* 
	PLL
	
   - Main clock F_OSC=14,7MHz @ Olimex LPC-P2106) [limits: 10 MHz to 25 MHz]
   - System should run at max. Frequency (60MHz) [limit: max 60 MHz]
   - Choose multiplier M=4 
     so cclk = M * F_OSC= 4 * 14745000Hz = 58980000 Hz
   - MSEL-Bit in PLLCFG (bits 0-4) MSEL = M-1
   - F_CCO must be inbetween the limits 156 MHz to 320 MHz
     datasheet: F_CCO = F_OSC * M * 2 * P
   - choose devider P=2 => F_CCO = 14745000Hz * 4 * 2 * 2
     = 235920000 ~=236 MHz
   - PSEL0 (Bit5 in PLLCFG) = 1, PSEL1 (Bit6) = 0 (0b01)
*/
#define FOSC            (14745600)  	// Master Oscillator Freq.
#define PLL_M		(4)         	// PLL Multiplier
#define MSEL		(PLL_M-1)
#define PSEL0 		5
#define PSEL1 		6
#define CCLK                (FOSC * PLL_M) 		// CPU Clock Freq.
#define PCLK                (CCLK / VPBDIV_VAL) 	// Pheripheal Bus Clock Freq.

#define PLLCON_PLLE	1
#define PLLCON_PLLC	2
#define	PLLSTAT_LOCK	BIT(10)

/*
	VPB (V... Pheriphal Bus)
	- choosen: VPB should run at full speed -> devider VPBDIV=1
	=> pclk = cclk = 59MHz
*/
#define VPBDIV_VAL	1

void SYSTEM_Init(void);


#endif
