#include "lpc210x.h"
#include "uart.h"

//void uart0ISR (void) __attribute__ ((interrupt ("IRQ")));


#define IRQ_MASK 0x00000080

static inline unsigned __get_cpsr(void)
{
  unsigned long retval;
  asm volatile (" mrs  %0, cpsr" : "=r" (retval) : /* no inputs */  );
  return retval;
}

static inline void __set_cpsr(unsigned val)
{
  asm volatile (" msr  cpsr, %0" : /* no outputs */ : "r" (val)  );
}

unsigned enableIRQ(void)
{
  unsigned _cpsr;

  _cpsr = __get_cpsr();
  __set_cpsr(_cpsr & ~IRQ_MASK);
  return _cpsr;
}


void uart0ISR(void)
{
	unsigned char iid,buf;
	
	UART_0Putch('I');
	// loop until not more interrupt sources
	while (((iid = UART0_IIR) & UIIR_NO_INT) == 0)
	{
		// identify & process the highest priority interrupt
    		switch (iid & UIIR_ID_MASK)
      		{
      			case UIIR_RLS_INT:                // Receive Line Status
        			UART0_LSR;                          // read LSR to clear
        			break;

      			case UIIR_CTI_INT:                // Character Timeout Indicator
      			case UIIR_RDA_INT:                // Receive Data Available
        			do
          			{				//CODE HERE
					UART0_RBR;

       				}
	       			while (UART0_LSR & ULSR_RDR);
				break;
      			default:                          // Unknown
        			UART0_LSR;
        			UART0_RBR;
        			break;
      		}
    	}

	VICVectAddr = 0x00;
}


void UART_0Init(unsigned short baud, unsigned char mode, unsigned char fmode)
{
	// set port pins for UART0
	PINSEL0 = (PINSEL0 & ~U0_PINMASK) | U0_PINSEL;

	UART0_IER = 0x00;                         // disable all interrupts
	UART0_IIR;                                // clear interrupt ID
	UART0_RBR;                                // clear receive register
	UART0_LSR;                                // clear line status register

	// set the baudrate
	UART0_LCR = ULCR_DLAB_ENABLE; 	  // set divisor latches (DLAB)
	UART0_DLL = (unsigned char)baud;          // set for baud low byte
	UART0_DLM = (unsigned char)(baud >> 8);   // set for baud high byte

	// set the number of characters and other
	// user specified operating parameters
	UART0_LCR = (mode & ~ULCR_DLAB_ENABLE); // clear DLAB "on-the-fly"
	UART0_FCR = fmode;

	// initialize the interrupt vector
	VICIntSelect &= ~BIT(VIC_UART0);  	// UART0 selected as IRQ
	VICIntEnable |= BIT(VIC_UART0);    	// UART0 interrupt enabled
	VICVectCntl0 = VIC_ENABLE | VIC_UART0;
	VICVectAddr0 = (unsigned)uart0ISR;  // address of the ISR

	// enable receiver interrupts
	UART0_IER = UIER_ERBFI;

	//enableIRQ();


}


int UART_0Putch(unsigned char ch)
{
	while (!(UART0_LSR & ULSR_THRE))	// wait for TX buffer to empty
		continue;                       // also either WDOG() or swap()

	UART0_THR = (unsigned char)ch;  	// put char to Transmit Holding Register
	return (unsigned char)ch;      
}

int UART_0Getch(void)
{
  if (UART0_LSR & ULSR_RDR)                 // check if character is available
    return UART0_RBR;                       // return character

  return -1;
}
